#include "gfunc.h"
#include <stdio.h>
#include <math.h>

#ifdef SIGTRAP
#include <signal.h>
#endif

#define MAXPAIRS 1000
 
double *x,*y;
int    ini,*ntxt;
FILE   *in = NULL;

main(argc,argv)

short argc;
char *argv[];

{
 short b0,atype[2];
 int i,j,dfile,ipen,curmod,isym,icst,dec[2],dir[2];
 int err,reso,language,rtxt,height,o_height;
 int owncolor,tcol,r[16],g[16],b[16];
 double xframe,yframe,xpic,ypic,xoff,yoff,xmax,ymax,xmin,ymin;
 double width,lgt[2],unit[4],fx,fy;
 char string[72],font[20],pos[2],coord[3],axt[6];
 char *calloc(),*fgets(),*strcpy();
 static char o_font[20];

#ifdef SIGTRAP
 int cleanup();
 signal(SIGFPE,cleanup);
 signal(SIGILL,cleanup);
 signal(SIGSEGV,cleanup);
#endif

 if (argc < 3)
 {
  printf("\nUSAGE: graph <infile> <dfile1> [ ... dfilen]\n\n");
  exit(0);
 }
 dfile = argc-2;
 
 ini = 0;
 
 if ((in = fopen(argv[1],"r")) == NULL)
 {
  printf("\nERROR: Can't open input file <%s>\n\n",argv[1]);
  exit(0);
 }
 x = y = NULL;

/***********************************************************
 *  reso:        Bildschirmauflsung (0-3)                 *
 *  language:    Fehlerausgabe (1=englisch,0=deutsch)      *
 ***********************************************************/
 
 fscanf(in,"%ld %ld",&reso,&language);
 if (reso < 0 || reso > 3)
 {
  printf("\nERROR: Invalid resolution parameter <%d>\n\n",reso);
  goto end;
 }
 if (language != 0 && language != 1)
 {
  printf("\nERROR: Invalid language parameter <%d>\n\n",language);
  goto end;
 }
 
/***********************************************************
 *  xframe:  Rahmenbreite                                  *
 *  yframe:  Rahmenhhe                                    *
 *  xpic:    Bildbreite                                    *
 *  ypic:    Bildhhe                                      *
 *  xoff:    X-Versatz                                     *
 *  yoff:    Y-Versatz                                     *
 ***********************************************************/
 
 fscanf(in,"%lf %lf %lf %lf %lf %lf",&xframe,&yframe,&xpic,&ypic,&xoff,&yoff);
 if ((xpic+xoff) > xframe)
 {
  xpic = xpic*(xpic+xoff)/xframe;
  xoff /= xpic;
 }
 if ((ypic+yoff) > yframe)
 {
  ypic = ypic*(ypic+yoff)/yframe;
  yoff /= ypic;
 }

/***********************************************************
 *  xmin:   Wertebereich x-Achse Minimum                   *
 *  xmax:   Wertebereich x-Achse Maximum                   *
 *  ymin:   Wertebereich y-Achse Minimum                   *
 *  ymax:   Wertebereich y-Achse Maximum                   *
 ***********************************************************/
 
 fscanf(in,"%lf %lf %lf %lf",&xmin,&xmax,&ymin,&ymax);
 if (xmin > xmax || ymin > ymax)
 {
  printf("\nERROR: Incorrect region range\n\n");
  goto end;
 }
 
/***********************************************************
 *  Skalierung der Achsen LIN = Linear                     *
 *                        LOG = Logarithmisch              *
 *                                                         *
 *  axt[0...2] : x-Achse                                   *
 *  axt[3...5] : y-Achse                                   *
 ***********************************************************/

 fscanf(in,"%s",axt);
 if ( strncmp(axt,"LIN",3) != 0 && strncmp(axt,"LOG",3) != 0) 
 {
  printf("\nERROR: Invalid axis specification <%s>\n\n",axt);
  goto end;
 }
 if ( strncmp(&axt[3],"LIN",3) != 0 && strncmp(&axt[3],"LOG",3) != 0) 
 {
  printf("\nERROR: Invalid axis specification <%s>\n\n",axt);
  goto end;
 }
 
 if (axt[1] == 'I')
  atype[0] = 0;
 else
  atype[0] = 1;
 if (axt[4] == 'I')
  atype[1] = 0;
 else
  atype[1] = 1;
   
   
/***********************************************************
 *  Richtung der Achsenticks                               *
 *                                                         *
 *  -1: Ticks nach innen                                   *
 *   1: Ticks nach aussen                                  *
 *                                                         *
 *   dir[0]: x-Achse                                       *
 *   dir[1]: y-Achse                                       *
 ***********************************************************/

 fscanf(in,"%ld %ld",&dir[0],&dir[1]);
 for (i = 0; i < 2; i++)
  if (dir[i] != 1 && dir[i] != -1)
  {
   printf("\nERROR: Invalid tick direction <%d>\n\n",dir[i]);
   goto end;
  }

/***********************************************************
 *  Lnge der Achsenticks                                  *
 *                                                         *
 *  lgt[0]:  x-Achse                                       *
 *  lgt[1]:  y-Achse                                       *
 *                                                         *
 *  Positionen der Achsenticks                             *
 *                                                         *
 *  unit[0...1]: x-Achse                                   *
 *  unit[2...3]: y-Achse (Siehe bei linax bzw. logax)      *
 *                                                         *
 *  Anzahl Dezimalstellen fr Beschriftung                 *
 *                                                         *
 *  dec[0]: x-Achse                                        *
 *  dec[1]: y-Achse                                        *
 ***********************************************************/
 
 fscanf(in,"%lf %lf",&lgt[0],&lgt[1]);
 fscanf(in,"%lf %lf %lf %lf",&unit[0],&unit[1],&unit[2],&unit[3]);
 fscanf(in,"%ld %ld",&dec[0],&dec[1]);
 for (i = 0; i < 2; i++)
  if (dec[i] < 0 || dec[i] > 10)
  {
   printf("\nERROR: Invalid decimal specification <%d>\n\n",dec[i]);
   goto end;
  }
  
/***********************************************************
 *  Eigene Farbpalette                                     *
 *                                                         *
 *  owncolor ( <= 0 Keine,  > 0 Eigene                     *
 *                                                         *
 *  Bei eigener Farbpalette                                *
 *                                                         *
 *  r[i], g[i], b[i]  RGB-Anteile von Farbe i (i = 0...15) *
 ***********************************************************/
 
  fscanf(in,"%ld",&owncolor);
  if (owncolor > 0)
  {
   for (i = 0; i < 16; i++)
    fscanf(in,"%ld %ld %ld",&r[i],&g[i],&b[i]);
  }
  
/***********************************************************
 *  Texte im Bild                                          *
 *                                                         *
 *  rtxt:  Anzahl der Textstellen                          *
 *                                                         *
 *  Wenn rtxt > 0:                                         *
 *                                                         *
 *  rtxt Zahlen ntxt[0...rtxt]: Textoperationen            *
 ***********************************************************/
 
  ntxt = NULL;
  fscanf(in,"%ld",&rtxt);

  if (rtxt > 0)
  {
   ntxt = (int *)calloc(rtxt,sizeof(int));
   if (ntxt == NULL)
   {
    printf("\nERROR: Can't allocate temporary memory\n\n");
    goto end0;
   }
   for (i = 0; i < rtxt; i++)
    fscanf(in,"%ld",&ntxt[i]);
  }
  
  /*******************************************************************/
  
  err = initialize(reso,language);
  if (err != 0) goto end0;
  ini = 1;
   
  frame(xframe,yframe);
  picture(xpic,ypic,xoff,yoff);
  region(xmin,xmax,ymin,ymax);
  err = mkwind();
  if (err != 0) goto end0;

  if (owncolor > 0)
  {
   for (i = 0; i < 16; i++)
    setrgb(i,r[i],g[i],b[i]);
  }

  if ((err = setfmt(dec[0],dec[1])) != 0) goto end0;
  
  switch(atype[0])
  {
   case 0:
    err = linax("XL",dir[0],lgt[0],unit[0],1);
    if (err != 0) goto end0;
    err = linax("XL",dir[0],lgt[0]/2.0,unit[1],0);
    if (err != 0) goto end0;
    err = linax("XU",-1,lgt[0],unit[0],0);
    if (err != 0) goto end0;
    break;
   case 1:
    err = logax("XL",dir[0],lgt[0],1,1);
    if (err != 0) goto end0;
    err = logax("XL",dir[0],lgt[0]/2.0,((int)unit[1]),0);
    if (err != 0) goto end0;
    err = logax("XU",-1,lgt[0],1,0);
    if (err != 0) goto end0;
  }
  switch(atype[1])
  {
   case 0:
    err = linax("YL",dir[0],lgt[0],unit[2],1);
    if (err != 0) goto end0;
    err = linax("YL",dir[0],lgt[0]/2.0,unit[3],0);
    if (err != 0) goto end0;
    err = linax("YU",-1,lgt[0],unit[2],0);
    if (err != 0) goto end0;
    break;
   case 1:
    err = logax("YL",dir[0],lgt[0],1,1);
    if (err != 0) goto end0;
    err = logax("YL",dir[0],lgt[0]/2.0,((int)unit[3]),0);
    if (err != 0) goto end0;
    err = logax("YU",-1,lgt[0],1,0);
    if (err != 0) goto end0;
  }

/***********************************************************
 *  Fr jede Textstelle                                    *
 *                                                         *
 *  coord: Koordinatensystem (FRA PIC REG REF)             *
 *  tcol:  Farbe (1...12)                                  *
 *                                                         *
 *  Fr jede Textoperation                                 *
 *                                                         *
 *  font:    Font                                          *
 *  height:  Fonthhe                                      *
 *  string:  Zu schreibende Zeichenkette                   *
 ***********************************************************/
 
 if (rtxt > 0)
 {
  strcpy(o_font,"topaz.font");
  o_height = 8;
  pwatch("OF");
  
  for (i = 0; i < rtxt; i++)
  {
   fscanf(in,"%s %ld",coord,&tcol);
   if ((err = setmode(coord)) != 0) goto end0;
   setpen(tcol);
   for (j = 0; j < ntxt[i]; j++)
   {
    fscanf(in,"%s %ld",font,&height);
    if ((err = strcmp(o_font,font)) != 0 || height != o_height)
    {
     if ((err = setfont(font,height)) != 0) goto end0;
     strcpy(o_font,font);
     o_height = height;
    }
    fscanf(in,"%lf %lf %2s",&fx,&fy,pos);
    fgets(string,72,in);
    b0 = 1;
    if (string[0] == '\n') 
    {
     fgets(string,72,in);
     b0 = 0;
    }
    string[strlen(string)-1] = '\0';
    if ((err = htext(fx,fy,pos,&string[b0])) != 0) goto end0;
    if (j == 0) setmode("REF");
   }
  }
 }
 fclose(in);
  
 setmode("REG");
 pwatch("ON");
 
 if ((x = (double *)calloc(MAXPAIRS,sizeof(double))) == NULL)
 {
  printf("\nERROR: Can't allocate temporary memory\n\n");
  goto end0;
 }
 if ((y = (double *)calloc(MAXPAIRS,sizeof(double))) == NULL)
 {
  printf("\nERROR: Can't allocate temporary memory\n\n");
  goto end0;
 }

 for (i = 0; i < dfile; i++)
 {
  if ((in = fopen(argv[i+2],"r")) == NULL)
  {
   printf("\nWARNING: Can't open <%s>\n\n",argv[i+2]);
  }
  else
  {
   fscanf(in,"%ld %ld %ld",&icst,&curmod,&ipen);
   if (icst == 5)
    fscanf(in,"%lf",&width);

   j = 0;
   while ((fscanf(in,"%lf %lf",&x[j],&y[j])) != EOF && j++ < MAXPAIRS);
   
   if (in != NULL) fclose(in);

   if (icst != 5) setdash(curmod);
   setpen(ipen);
  
   switch(icst)
   {
    case 0:
     polygon(x,y,j);
     break;
    case 1:
     sncurve(j,x,y);
     break;
    case 2:
     srcurve(j,x,y);
     break;
    case 3:
     spcurve(j,x,y);
     break;
    case 4:
     stcurve(j,x,y);
     break;
    case 5:
     setmark(curmod,width,x,y,j);
     break;
    default:
     printf("\nERROR: Invalid mode parameter <%d> (0-4)\n\n",icst);
   }
  }
 }
  
 end0:
 
 if (ntxt != NULL) free(ntxt);
 if (x != NULL) free(x);
 if (y != NULL) free(y);
 finish();
  
 end:
 
 if (in != NULL) fclose(in);
}

#ifdef SIGTRAP
cleanup(sig)

int sig;
{
 if (ini == 1)
  finish();
  
 if (ntxt != NULL) free(ntxt);
 if (x != NULL) free(x);
 if (y != NULL) free(y);
 if (in != NULL) fclose(in);
 printf("\nFATAL ERROR: Processor trap No <%d> ... terminating\n\n",sig);
 exit(0);
} 
#endif
